/**
    \file sigslot_presentation.cpp
    \author Kurt Stege

    Brief demonstration of the Sigslot concept.

    These examples are written to demonstrate the usage
    of the Sigslot concept.

    Most classes are formatted in a way that makes them
    usable to be copied into a powerpoint slides presentation.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-09-08 (ks):    File created.
    2005-09-08 (ks):    Current version.
*/

#include <stdio.h>

#ifdef _MSC_VER
// #pragma warning(disable:4786) // warning C4786: identifier was truncated to '255' characters in the debug information
// #pragma warning(disable:4355) // warning C4355: 'this' : used in base member initializer list
#endif


// ******************************************************************
// ******************************************************************
// ******************************************************************

#if 0 // invalid notation
/*
    I would like to use this notation.
    slot or NSigslot::slot could be typedefd to void.
    But how to change the signal to a Sigslot-Signal?
*/

#include "Sigslot/TSignature.hpp"

class CConvert
{
    slot   input  (double speed_in_mps);
    signal output (double speed_in_kmh);
};

/*!
    Gets a speed value in meters per second,
    converts it to kilometers per hour, and
    fires the output signal with the result.
*/

slot CConvert::input (double speed_in_mps)
{
    output (speed_in_mps * 3.6);
}

#endif // invalid notation


// ******************************************************************
// ******************************************************************
// ******************************************************************

#include "Sigslot/TSignature.hpp"

typedef NSigslot::TSignature1<double> tSig_speed_in_kmh;

class CConvert
{
    public:

    void SLOT_input  (double speed_in_mps);
    tSig_speed_in_kmh::CSignal SIGNAL_output;
};

/*!
    Gets a speed value in meters per second,
    converts it to kilometers per hour, and
    fires the output signal with the result.
*/
void CConvert::SLOT_input (double speed_in_mps)
{
    SIGNAL_output (speed_in_mps * 3.6);
}


// ******************************************************************

typedef NSigslot::TSignature1<double> tSig_speed_in_mps;

class CSensor
{
    public:

    tSig_speed_in_mps::CSignal SIGNAL_mps;

    void measureSpeed();
};

void CSensor::measureSpeed()
{
    // This demo assumes 23 miles per hour.

    SIGNAL_mps (23.0);
}


// ******************************************************************

#include <iostream>

class CPrintKMH
{
    public:

    void SLOT_kmh (double speed_in_kmh);
};

void CPrintKMH::SLOT_kmh (double speed_in_kmh)
{
    std::cout << "The speed is " << speed_in_kmh << " kmh.\n";
}


// ******************************************************************


class CPrintMPS
{
    public:

    void SLOT_mps (double speed_in_mps);
};

void CPrintMPS::SLOT_mps (double speed_in_mps)
{
    std::cout << "The speed is " << speed_in_mps << " meters per second.\n";
}


// ******************************************************************
/*!
    The application \c CApp1 takes four components and wires
    them together using two cable functions.
*/

class CApp1
{
    private:

    // The components:

    CSensor     m_sensor;
    CConvert    m_convert;
    CPrintMPS   m_printMPS;
    CPrintKMH   m_printKMH;

    // The cables:

    static void cable_sensor_mps     (void *context, const tSig_speed_in_mps::CMessage &msg);
    static void cable_convert_output (void *context, const tSig_speed_in_kmh::CMessage &msg);

    public:

    void run_application();
};

void CApp1::run_application()
{
    // plug the cables into the signals

    m_sensor.SIGNAL_mps     = tSig_speed_in_mps::CSignal(cable_sensor_mps, this);
    m_convert.SIGNAL_output = tSig_speed_in_kmh::CSignal(cable_convert_output, this);

    // let the application do some application specific work

    m_sensor.measureSpeed();
}


void CApp1::cable_sensor_mps (void *context, const tSig_speed_in_mps::CMessage &msg)
{
    CApp1 *this_ = static_cast<CApp1 *>(context);
    this_->m_printMPS.SLOT_mps(msg.m_par1);
    this_->m_convert.SLOT_input(msg.m_par1);
}

void CApp1::cable_convert_output (void *context, const tSig_speed_in_kmh::CMessage &msg)
{
    CApp1 *this_ = static_cast<CApp1 *>(context);
    this_->m_printKMH.SLOT_kmh(msg.m_par1);
}


// ******************************************************************
// ******************************************************************
// ******************************************************************

void run_sigslot_presentation()
{
    CApp1 app1;

    app1.run_application();
}


// ******************************************************************
// ******************************************************************
// ******************************************************************
